#include "iostream.h"
#include "AtlasProd.h"
#include "histo1D.h"
#include "HepODBMS/tagdb/HepTagDbApplication.h"
#include "HepODBMS/tagdb/HepExplorableGenericTags.h"
#include "HepODBMS/tagdb/HepExplorableAttribute.h"


class myAtlasApplication : public HepTagDbApplication {
private :
   void readGenericTags(int npart, int hist); 
public:
   myAtlasApplication() {};
   virtual int clusteringInit();
   void run(int npart, int hist);
};

void myAtlasApplication::readGenericTags(int npart, int hist) 
{
   HepRef(AtlasProdEvent) evt;
   HepRef(AtlasProdTag) theRealTag;

   // Retrieve the collection of real tags
   HepExplorable *expCollect = HepExplorable::findExplorable("AtlasProdTag");
   if (!expCollect)
      fatal("could not find AtlasProdTag");

   HepDatabaseRef  histoDb  = db("HistoDB");
   HepRef(Histo1D) pmass = new(histoDb) Histo1D("Particle Mass", 100, 0., 15.);
   HepRef(Histo1D) tmass = new(histoDb) Histo1D("Total Mass of Event",
                                                100, 0., 600.);
   HepRef(Histo1D) hpart = new(histoDb) Histo1D("Number of particles",
                                                100, 0., 1000.);

   // Start main loop over all events
   float mass, totmass;
   int cnt = 0, sel = 0;
   for (int rc = expCollect->start(); rc != 0 ; rc = expCollect->next()) {

      expCollect->getCurrentObject(theRealTag);

      if (npart == -1 || theRealTag->Npart < npart) {
         evt = theRealTag->getEvent();
         totmass = 0;
         for (int i = 0; i < theRealTag->Npart; i++) {
            mass = evt->particles[i].mass;
            if (hist) pmass->fill(mass);
            totmass += mass;
         }
         if (hist) tmass->fill(totmass);
         sel++;
      }
      if (hist) hpart->fill(theRealTag->Npart);

      cnt++;
      if (!(cnt % 100))
         printf("Processing event %d\n", cnt);
   } 

   printf("%f percent of events accepted (%d of %d)\n", 
          sel/(float)cnt*100., sel, cnt);

   printf("number of entries in pmass %d\n", pmass->get_entries_number());
   printf("number of entries in tmass %d\n", tmass->get_entries_number());
   printf("number of entries in hpart %d\n", hpart->get_entries_number());
}

int myAtlasApplication::clusteringInit()
{
   // Call base class first
   if (HepTagDbApplication::clusteringInit()) {
      AtlasProdEvent::clustering = HepEvent::clustering;
      AtlasProdTag::clustering   = HepEventTag::clustering;      
   }
   return 1;
}

void myAtlasApplication::run(int npart, int hist) 
{
   Init();  // Session init & clustering hints
   if (hist)
      startUpdate();
   else
      startRead();
   readGenericTags(npart, hist);
   commit();
}

int main(int argc, char *argv[]) 
{
   // By default (-1) calculates totmass for all events,
   // otherwise only for those events that have <npart particles
   int npart = -1;
   if (argc >= 2)
      npart = atoi(argv[1]);

   // By default (1) histogram, to not histogram set to 0
   int hist = 1;
   if (argc == 3)
      hist = atoi(argv[2]);

   myAtlasApplication mApp;

   mApp.run(npart, hist);

   return 0;
}
