#include "iostream.h"
#include "AtlasProd.h"
#include "HepODBMS/tagdb/HepTagDbApplication.h"
#include "HepODBMS/tagdb/HepExplorableGenericTags.h"
#include "HepODBMS/tagdb/HepExplorableAttribute.h"

#include <TROOT.h>
#include <TFile.h>
#include <TH1.h>

TROOT root("objy", "Atlas Objy and ROOT");


class myAtlasApplication : public HepTagDbApplication {
private :
   void readGenericTags(int npart, int histo); 
public:
   myAtlasApplication() {};
   virtual int clusteringInit();
   void run(int npart, int histo);
};

void myAtlasApplication::readGenericTags(int npart, int histo) 
{
   HepRef(AtlasProdEvent) evt;
   HepRef(AtlasProdTag) theRealTag;

   // Retrieve the collection of real tags
   HepExplorable *expCollect = HepExplorable::findExplorable("AtlasProdTag");
   if (!expCollect)
      fatal("could not find AtlasProdTag");

   TFile hfile("atlas.root", "recreate", "Atlas ROOT Histograms");

   TH1F pmass("pmass", "Particle Mass", 100, 0., 15.);
   TH1F tmass("tmass", "Total Mass of Event", 100, 0., 600.);
   TH1F hpart("hpart", "Number of particles", 100, 0., 1000.);

   // Start main loop over all events
   float mass, totmass;
   int cnt = 0, sel = 0;
   for (int rc = expCollect->start(); rc != 0 ; rc = expCollect->next()) {

      expCollect->getCurrentObject(theRealTag);

      if (npart == -1 || theRealTag->Npart < npart) {
         evt = theRealTag->getEvent();
         totmass = 0;
         for (int i = 0; i < theRealTag->Npart; i++) {
            mass = evt->particles[i].mass;
            if (histo) pmass.Fill(mass);
            totmass += mass;
         }
         if (histo) tmass.Fill(totmass);
         sel++;
      }
      if (histo) hpart.Fill(theRealTag->Npart);

      cnt++;
      if (!(cnt % 100))
         printf("Processing event %d\n", cnt);
   } 

   printf("%f percent of events accepted (%d of %d)\n", 
          sel/(float)cnt*100., sel, cnt);

   printf("number of entries in pmass %f\n", pmass.GetEntries());
   printf("number of entries in tmass %f\n", tmass.GetEntries());
   printf("number of entries in hpart %f\n", hpart.GetEntries());

   if (histo) hfile.Write();
}

int myAtlasApplication::clusteringInit()
{
   // call base class first
   if (HepTagDbApplication::clusteringInit()) {
      AtlasProdEvent::clustering = HepEvent::clustering;
      AtlasProdTag::clustering   = HepEventTag::clustering;      
   }
   return 1;
}

void myAtlasApplication::run(int npart, int histo) 
{
   Init();  // Session init & clustering hints
   startRead();
   readGenericTags(npart, histo);
   commit();
}

int main(int argc, char *argv[]) 
{
   // by default (-1) calculate totmass for all events,
   // otherwise only for those events that have <npart particles
   int npart = -1;
   if (argc >= 2)
      npart = atoi(argv[1]);

   // by default (1) histogram, to not histogram set to 0
   int histo = 1;
   if (argc == 3)
      histo = atoi(argv[2]);

   myAtlasApplication mApp;

   mApp.run(npart, histo);

   return 0;
}
