#ifndef __CINT__
#include <stdlib.h>

#include <TROOT.h>
#include <TFile.h>
#include <TTree.h>
#include <TH1.h>

#include "AtlasProd.h"
#endif


void totalmass(int npart = -1, int histo = 1)
{
#ifdef __CINT__
   gROOT->Reset();
   gSystem->Load("libAtlasProd.so");
#endif

   // Connect ROOT event database
   TFile db("/shift/shd09/data10/vp/users/rdm/rtdb/event.root");

   // Get TTree container from DB and setup clustering hints
   TTree *h1 = (TTree*)db.Get("Event");
   AtlasProdEvent *event = new AtlasProdEvent;

   h1->SetBranchAddress("event", &event);

   h1->SetBranchStatus("*", 0);               // disable all branches
   h1->SetBranchStatus("Npart", 1);           // activate named branch
   h1->SetBranchStatus("particles.mass", 1);  // activate named branch

   TBranch *bNpart = (TBranch *) h1->GetBranch("Npart");

   // Open histogram output file and create histograms
   TFile hfile("atlas.root", "recreate", "Atlas ROOT Histograms", 1);

   TH1F pmass("pmass", "Particle Mass", 100, 0., 15.);
   TH1F tmass("tmass", "Total Mass of Event", 100, 0., 600.);
   TH1F hpart("hpart", "Number of particles", 100, 0., 1000.);

   // Start main loop over all events
   Float_t mass, totmass;
   int cnt = 0, sel = 0;
   Int_t nentries = (int)h1->GetEntries();
   for (int i = 0; i < nentries; i++) {

      if (npart != -1)
         bNpart->GetEvent(i);

      if (npart == -1 || event->Npart < npart) {
         h1->GetEvent(i);
         totmass = 0;
         for (int j = 0; j < event->Npart; j++) {
            mass = event->GetParticle(j)->mass;
            if (histo) pmass.Fill(mass);
            totmass += mass;
         }
         if (histo) tmass.Fill(totmass);
         sel++;
      }
      if (histo) hpart.Fill(event->Npart);

      cnt++;
      if (!(cnt % 100))
         printf("Processing event %d\n", cnt);
   }

   printf("%f percent of events accepted (%d of %d)\n",
          sel/(float)cnt*100., sel, cnt);

   printf("number of entries in pmass %f\n", pmass.GetEntries());
   printf("number of entries in tmass %f\n", tmass.GetEntries());
   printf("number of entries in hpart %f\n", hpart.GetEntries());

   // Write histograms to database
   if (histo) hfile.Write();
}


#ifndef __CINT__

int main(int argc, char *argv[])
{
   // Initialize ROOT system
   TROOT root("AtlasProd", "AtlasProd test");

   // By default (-1) calculates totmass for all events,
   // otherwise only for those events that have <npart particles
   int npart = -1;
   if (argc >= 2)
      npart = atoi(argv[1]);

   // By default (1) histogram, to not histogram set to 0
   int histo = 1;
   if (argc == 3)
      histo = atoi(argv[2]);

   totalmass(npart, histo);

   return 0;
}

#endif

